#include <xc.h>
#include "flash.h"



/*************************************************************************
* Func: Required_Write_Sequence - ASSUMING GIE = 0 for BOOTLOADER
* Desc: Required Sequence to write to Flash
*************************************************************************/
void Required_Write_Sequence(void)
{
  EECON2 = 0x55;                                                                // Enter write sequence
  EECON2 = 0xAA;
  EECON1bits.WR = 1;                                                            // Perform Write
  NOP();
  NOP();
  while(EECON1bits.WR);                                                         // Wait till the write completion
  EECON1bits.WREN = 0;                                                          // Disable writes
}



/*************************************************************************
* Func: Erase_Flash_Segment
* Desc: Erases Flash Segment (32 bytes)
*************************************************************************/
void Erase_Flash_Segment(unsigned short SegmentN)
{
  EEADR = SegmentN*FLASHSEGMENTSIZE + FLASHSTORAGEPOSITION;                     // Get Address Based on Segment Number
  
  EECON1bits.CFGS = 0;                                                          // Access Flash or Eeprom
  EECON1bits.EEPGD = 1;                                                         // Access Flash Memory (Pag 107)
  EECON1bits.FREE = 1;                                                          // Next Operation is Erase
  EECON1bits.WREN = 1;                                                          // Allow Flash Writing
  EECON1bits.LWLO = 1;                                                          // Only load Latches
  Required_Write_Sequence();                                                    // Perform Erase
}



/*************************************************************************
* Func: Save_Flash_Chunk
* Desc: Saves Flash Chunk from RAM Buffer
*************************************************************************/
void Save_Flash_Chunk(unsigned short ChunkN, unsigned char* VChunk)
{
unsigned char i;
unsigned short badd;
  badd =  ChunkN*FLASHCHUNKSIZE + FLASHSTORAGEPOSITION;                         // Get Address Based on Chunk Number
  
  EECON1bits.EEPGD = 1;                                                         // Access Flash Memory (Pag 107)
  EECON1bits.WREN = 1;                                                          // Allow Flash Writing
  EECON1bits.CFGS = 0;                                                          // Access Flash or Eeprom
  EECON1bits.LWLO = 1;                                                          // Only load Latches
  EEADR = badd;
  
  for (i=0;i!=FLASHCHUNKSIZE;i++)                                               // Preload Holding Registers to be written to Flash
  {
    EEDATL = *(VChunk+i*2);                                                     // Load VChunk Little-Endian
    EEDATH = *(VChunk+i*2+1); 
    EECON2=0x55;
    EECON2=0xAA;
    EECON1bits.WR=1;
    NOP();
    NOP();
    EEADR++;
  }
  EEDATL = *(VChunk);      
  EEDATH = *(VChunk+1);  
  EECON1bits.LWLO=0;                                                            // Next operation will perform Write
  EEADR=badd;
  Required_Write_Sequence();                                                    // Perform Block Write
}
